@@ -15,7 +15,6 @@ try:
 except ImportError:
     import xml.etree.ElementTree as et
 
-
 import os
 import re
 
@@ -29,8 +28,9 @@ import traceback
 VERSION_STRING = '0.0.1'
 
 from pyqtconfig import ConfigManager
-
-import os,sys,time
+import os
+import sys
+import time
 
 try:
     from IPython.kernel import KernelManager
@@ -44,7 +44,6 @@ from IPython.nbconvert.exporters.export import exporter_map as IPyexporter_map
 from IPython.utils.ipstruct import Struct
 from runipy.notebook_runner import NotebookRunner
 
-
 _w = None
 
 MODE_MANUAL = 0
@@ -52,6 +51,7 @@ MODE_WATCH_FILES = 1
 MODE_WATCH_FOLDER = 2
 MODE_TIMER = 3
 
+
 class NotebookNotFound(Exception):
     pass
 
@@ -65,6 +65,7 @@ def mkdir_p(path):
         else:
             raise
 
+
 class Logger(logging.Handler):
     def __init__(self, parent, widget, out=None, color=None):
         super(Logger, self).__init__()
@@ -82,25 +83,22 @@ class Logger(logging.Handler):
     def emit(self, record):
         msg = self.format(record)
 
-        item = QTreeWidgetItem()
-        item.setText(0, record.name)
-        item.setText(1, msg)
-
-        bg = {
+        color = {
             logging.CRITICAL: QColor(164, 0, 0, 50),
             logging.ERROR: QColor(239, 41, 41, 50),
             logging.WARNING: QColor(252, 233, 79, 50),
-            logging.INFO: None,
+            logging.INFO: QColor(0, 0, 0, 255),
             logging.DEBUG: QColor(114, 159, 207, 50),
-            logging.NOTSET: None,
+            logging.NOTSET: QColor(0, 0, 0, 255),
         }[record.levelno]
-        if bg:
-            for c in range(3):
-                item.setBackground(c, QBrush(bg))
-
-        self.widget.addTopLevelItem(item)
-        self.widget.scrollToBottom()
 
+        rows = self.widget._current_rows
+        if rows:
+            rows = rows[-50:]
+        rows.append('<div style="color:%s;">%s</div>' % ( color.name(), msg.replace('\n','<br />') ) )
+        self.widget._current_rows = rows
+        self.widget.setHtml('<html><body>' + '\n\n\n'.join(rows) + '</body></html>')
+        
     def write(self, m):
         pass
 
@@ -138,48 +136,46 @@ class GenericDialog(QDialog):
 
         # Set dialog layout
         self.setLayout(self.sizer)
-        
+
 
 class AutomatonDialog(GenericDialog):
-    
+
     mode_options = {
         'Manual': MODE_MANUAL,
         'Watch files': MODE_WATCH_FILES,
         'Watch folder': MODE_WATCH_FOLDER,
         'Timer': MODE_TIMER,
     }
-    
-    
+
     def __init__(self, parent, **kwargs):
         super(AutomatonDialog, self).__init__(parent, **kwargs)
         self.setWindowTitle("Edit Automaton")
-    
+
         self.config = ConfigManager()
 
         gb = QGroupBox('IPython notebook(s) (*.ipynb)')
         grid = QGridLayout()
         notebook_path_le = QLineEdit()
-        self.config.add_handler('notebook_paths', notebook_path_le, mapper=(lambda x:x.split(";"), lambda x:";".join(x)))
-        grid.addWidget(notebook_path_le, 0, 0, 1,2)
+        self.config.add_handler('notebook_paths', notebook_path_le, mapper=(lambda x: x.split(";"), lambda x: ";".join(x)))
+        grid.addWidget(notebook_path_le, 0, 0, 1, 2)
 
         notebook_path_btn = QToolButton()
-        notebook_path_btn.setIcon( QIcon(os.path.join(utils.scriptdir, 'icons', 'document-attribute-i.png')) )
-        notebook_path_btn.clicked.connect( lambda: self.onNotebookBrowse(notebook_path_le) )
-        grid.addWidget(notebook_path_btn, 0, 2, 1,1)
+        notebook_path_btn.setIcon(QIcon(os.path.join(utils.scriptdir, 'icons', 'document-attribute-i.png')))
+        notebook_path_btn.clicked.connect(lambda: self.onNotebookBrowse(notebook_path_le))
+        grid.addWidget(notebook_path_btn, 0, 2, 1, 1)
         gb.setLayout(grid)
-        
+
         self.layout.addWidget(gb)
 
         gb = QGroupBox('Automaton mode')
         grid = QGridLayout()
         mode_cb = QComboBox()
-        mode_cb.addItems( self.mode_options.keys() )
+        mode_cb.addItems(self.mode_options.keys())
         mode_cb.currentIndexChanged.connect(self.onChangeMode)
         self.config.add_handler('mode', mode_cb, mapper=self.mode_options)
         grid.addWidget(QLabel('Mode'), 0, 0)
         grid.addWidget(mode_cb, 0, 1)
 
-
         grid.addWidget(QLabel('Hold trigger'), 1, 0)
         fwatcher_hold_sb = QSpinBox()
         fwatcher_hold_sb.setRange(0, 60)
@@ -189,129 +185,127 @@ class AutomatonDialog(GenericDialog):
         gb.setLayout(grid)
 
         self.layout.addWidget(gb)
-        
+
         self.watchfile_gb = QGroupBox('Watch files')
         grid = QGridLayout()
 
         watched_path_le = QLineEdit()
-        grid.addWidget(watched_path_le, 0, 0, 1,2)
-        self.config.add_handler('watched_files', watched_path_le, mapper=(lambda x:x.split(";"), lambda x:";".join(x)))
-        
+        grid.addWidget(watched_path_le, 0, 0, 1, 2)
+        self.config.add_handler('watched_files', watched_path_le, mapper=(lambda x: x.split(";"), lambda x: ";".join(x)))
+
         watched_path_btn = QToolButton()
-        watched_path_btn.setIcon( QIcon(os.path.join(utils.scriptdir, 'icons', 'document-copy.png')) )
+        watched_path_btn.setIcon(QIcon(os.path.join(utils.scriptdir, 'icons', 'document-copy.png')))
         watched_path_btn.setStatusTip('Add file(s)')
-        watched_path_btn.clicked.connect( lambda: self.onFilesBrowse(watched_path_le) )
-        grid.addWidget(watched_path_btn, 0, 2, 1,1)
-        
+        watched_path_btn.clicked.connect(lambda: self.onFilesBrowse(watched_path_le))
+        grid.addWidget(watched_path_btn, 0, 2, 1, 1)
+
         grid.addWidget(QLabel('Watch window'), 1, 0)
         watch_window_sb = QSpinBox()
         watch_window_sb.setRange(0, 60)
         watch_window_sb.setSuffix(' secs')
         self.config.add_handler('watch_window', watch_window_sb)
         grid.addWidget(watch_window_sb, 1, 1)
-        
+
         self.watchfile_gb.setLayout(grid)
         self.layout.addWidget(self.watchfile_gb)
-        
+
         self.watchfolder_gb = QGroupBox('Watch folder')
         grid = QGridLayout()
 
         watched_path_le = QLineEdit()
-        grid.addWidget(watched_path_le, 0, 0, 1,2)
+        grid.addWidget(watched_path_le, 0, 0, 1, 3)
         self.config.add_handler('watched_folder', watched_path_le)
-        
+
         watched_path_btn = QToolButton()
-        watched_path_btn.setIcon( QIcon(os.path.join(utils.scriptdir, 'icons', 'folder-horizontal-open.png')) )
+        watched_path_btn.setIcon(QIcon(os.path.join(utils.scriptdir, 'icons', 'folder-horizontal-open.png')))
         watched_path_btn.setStatusTip('Add folder')
-        watched_path_btn.clicked.connect( lambda: self.onFolderBrowse(watched_path_le) )
-        grid.addWidget(watched_path_btn, 0, 2, 1,1)
+        watched_path_btn.clicked.connect(lambda: self.onFolderBrowse(watched_path_le))
+        grid.addWidget(watched_path_btn, 0, 3, 1, 1)
 
         grid.addWidget(QLabel('Iterate files in folder'), 3, 0)
         loop_folder_sb = QCheckBox()
-        self.config.add_handler('loop_watched_folder', loop_folder_sb)
+        self.config.add_handler('iterate_watched_folder', loop_folder_sb)
         grid.addWidget(loop_folder_sb, 3, 1)
 
-        
-        self.watchfolder_gb.setLayout(grid)
-        self.layout.addWidget(self.watchfolder_gb)        
+        loop_wildcard_le = QLineEdit()
+        self.config.add_handler('iterate_wildcard', loop_wildcard_le)
+        grid.addWidget(loop_wildcard_le, 3, 2)
 
+        self.watchfolder_gb.setLayout(grid)
+        self.layout.addWidget(self.watchfolder_gb)
 
         self.timer_gb = QGroupBox('Timer')
         grid = QGridLayout()
-        
+
         grid.addWidget(QLabel('Run every'), 0, 0)
         watch_timer_sb = QSpinBox()
         watch_timer_sb.setRange(0, 60)
         watch_timer_sb.setSuffix(' secs')
         self.config.add_handler('timer_seconds', watch_timer_sb)
         grid.addWidget(watch_timer_sb, 0, 1)
-        
+
         self.timer_gb.setLayout(grid)
         self.layout.addWidget(self.timer_gb)
 
-        self.manual_gb = QGroupBox('Manual') # No show
+        self.manual_gb = QGroupBox('Manual')  # No show
         grid = QGridLayout()
         grid.addWidget(QLabel('No configuration'), 0, 0)
         self.manual_gb.setLayout(grid)
         self.layout.addWidget(self.manual_gb)
-        
-        
+
         gb = QGroupBox('Output')
         grid = QGridLayout()
         output_path_le = QLineEdit()
         self.config.add_handler('output_path', output_path_le)
-        grid.addWidget(output_path_le, 0, 0, 1,2)
-        
+        grid.addWidget(output_path_le, 0, 0, 1, 2)
+
         notebook_path_btn = QToolButton()
-        notebook_path_btn.setIcon( QIcon(os.path.join(utils.scriptdir, 'icons', 'folder-horizontal-open.png')) )
-        notebook_path_btn.clicked.connect( lambda: self.onFolderBrowse(notebook_path_le) )
-        grid.addWidget(notebook_path_btn, 0, 2, 1,1)
+        notebook_path_btn.setIcon(QIcon(os.path.join(utils.scriptdir, 'icons', 'folder-horizontal-open.png')))
+        notebook_path_btn.clicked.connect(lambda: self.onFolderBrowse(notebook_path_le))
+        grid.addWidget(notebook_path_btn, 0, 2, 1, 1)
 
         export_cb = QComboBox()
-        export_cb.addItems( IPyexporter_map.keys() )
+        export_cb.addItems(IPyexporter_map.keys())
         self.config.add_handler('output_format', export_cb)
         grid.addWidget(QLabel('Notebook output format'), 1, 0)
         grid.addWidget(export_cb, 1, 1)
 
-
         gb.setLayout(grid)
 
-        self.layout.addWidget(gb)        
+        self.layout.addWidget(gb)
 
         self.layout.addStretch()
         self.finalise()
 
         self.onChangeMode(mode_cb.currentIndex())
-        
+
     def onNotebookBrowse(self, t):
         global _w
         filenames, _ = QFileDialog.getOpenFileNames(_w, "Load IPython notebook(s)", '', "IPython Notebooks (*.ipynb);;All files (*.*)")
         if filenames:
-            self.config.set('notebook_paths',filenames)
+            self.config.set('notebook_paths', filenames)
 
     def onFolderBrowse(self, t):
         global _w
         filename = QFileDialog.getExistingDirectory(_w, "Select folder to watch")
         if filename:
-            self.config.set('watched_folder',filename)
+            self.config.set('watched_folder', filename)
 
     def onFilesBrowse(self, t):
         global _w
         filenames, _ = QFileDialog.getOpenFileNames(_w, "Select file(s) to watch")
         if filenames:
-            self.config.set('watched_files',filenames)
-        
+            self.config.set('watched_files', filenames)
         
     def onChangeMode(self, i):
-        for m,gb in { MODE_MANUAL: self.manual_gb, MODE_WATCH_FILES: self.watchfile_gb, MODE_WATCH_FOLDER: self.watchfolder_gb, MODE_TIMER: self.timer_gb }.items():
+        for m, gb in {MODE_MANUAL: self.manual_gb, MODE_WATCH_FILES: self.watchfile_gb, MODE_WATCH_FOLDER: self.watchfolder_gb, MODE_TIMER: self.timer_gb}.items():
             if m == list(self.mode_options.items())[i][1]:
                 gb.show()
             else:
                 gb.hide()
         
-        
     def sizeHint(self):
-        return QSize(400,200)     
+        return QSize(400, 200)
 
 
 class AutomatonListDelegate(QAbstractItemDelegate):
@@ -320,34 +314,32 @@ class AutomatonListDelegate(QAbstractItemDelegate):
         # GET TITLE, DESCRIPTION AND ICON
         ic = QIcon(index.data(Qt.DecorationRole))
         automaton = index.data(Qt.UserRole)
-        
+
         f = QFont()
         f.setPointSize(10)
         painter.setFont(f)
 
         if automaton.is_running:
             painter.setPen(QPalette().text().color())
-            painter.fillRect(option.rect, QBrush(QColor(0,255,0,50) ))
+            painter.fillRect(option.rect, QBrush(QColor(0, 255, 0, 50)))
         elif automaton.latest_run['success'] == False:
             painter.setPen(QPalette().text().color())
-            painter.fillRect(option.rect, QBrush(QColor(255,0,0,50) ))
+            painter.fillRect(option.rect, QBrush(QColor(255, 0, 0, 50)))
         elif option.state & QStyle.State_Selected:
             painter.setPen(QPalette().highlightedText().color())
             painter.fillRect(option.rect, QBrush(QPalette().highlight().color()))
         else:
             painter.setPen(QPalette().text().color())
-
         
         r = QRect(5, 4, 12, 12)
         r.translate(option.rect.x(), option.rect.y())
-        icn = QIcon(os.path.join(utils.scriptdir, 'icons', 'document-attribute-i-sm.png') )
-        painter.drawPixmap(r, icn.pixmap(QSize(12,12)))
+        icn = QIcon(os.path.join(utils.scriptdir, 'icons', 'document-attribute-i-sm.png'))
+        painter.drawPixmap(r, icn.pixmap(QSize(12, 12)))
 
         r = QRect(5, 36, 12, 12)
         r.translate(option.rect.x(), option.rect.y())
-        icn = QIcon(os.path.join(utils.scriptdir, 'icons', 'disk-sm.png') )
-        painter.drawPixmap(r, icn.pixmap(QSize(12,12)))
-
+        icn = QIcon(os.path.join(utils.scriptdir, 'icons', 'disk-sm.png'))
+        painter.drawPixmap(r, icn.pixmap(QSize(12, 12)))
         #r = option.rect.adjusted(26, 5, 0, 0)
 
         pen = QPen()
@@ -359,126 +351,126 @@ class AutomatonListDelegate(QAbstractItemDelegate):
         painter.setPen(pen)
 
         # NOTEBOOK
-        r = QRect(20, 4, option.rect.width()-40, 20)
+        r = QRect(20, 4, option.rect.width() - 40, 20)
         r.translate(option.rect.x(), option.rect.y())
-        painter.drawText(r, Qt.AlignLeft, ";".join( automaton.config.get('notebook_paths') ))
+        painter.drawText(r, Qt.AlignLeft, ";".join(automaton.config.get('notebook_paths')))
 
         # OUTPUT
-        r = QRect(20, 36, option.rect.width()-40, 20)
+        r = QRect(20, 36, option.rect.width() - 40, 20)
         r.translate(option.rect.x(), option.rect.y())
         painter.drawText(r, Qt.AlignLeft, automaton.config.get('output_path'))
 
-
         # WATCH PATH
-        r = QRect(20, 20, option.rect.width()-40, 20)
+        r = QRect(20, 20, option.rect.width() - 40, 20)
         r.translate(option.rect.x(), option.rect.y())
         if automaton.config.get('mode') == MODE_WATCH_FILES:
-            painter.drawText(r, Qt.AlignLeft, ";".join( automaton.config.get('watched_files') ))
-            icn = QIcon(os.path.join(utils.scriptdir, 'icons', 'document-copy-sm.png') )
-            
+            painter.drawText(r, Qt.AlignLeft, ";".join(automaton.config.get('watched_files')))
+            icn = QIcon(os.path.join(utils.scriptdir, 'icons', 'document-copy-sm.png'))
+
         elif automaton.config.get('mode') == MODE_WATCH_FOLDER:
-            painter.drawText(r, Qt.AlignLeft, automaton.config.get('watched_folder') )
-            icn = QIcon(os.path.join(utils.scriptdir, 'icons', 'folder-horizontal-open-sm.png') )
-            
+            painter.drawText(r, Qt.AlignLeft, automaton.config.get('watched_folder'))
+            icn = QIcon(os.path.join(utils.scriptdir, 'icons', 'folder-horizontal-open-sm.png'))
+
         elif automaton.config.get('mode') == MODE_TIMER:
-            painter.drawText(r, Qt.AlignLeft, "%s seconds(s)" % automaton.config.get('timer_seconds') )
-            icn = QIcon(os.path.join(utils.scriptdir, 'icons', 'clock-select-sm.png') )
+            painter.drawText(r, Qt.AlignLeft, "%s seconds(s)" % automaton.config.get('timer_seconds'))
+            icn = QIcon(os.path.join(utils.scriptdir, 'icons', 'clock-select-sm.png'))
 
         elif automaton.config.get('mode') == MODE_MANUAL:
-            icn = QIcon(os.path.join(utils.scriptdir, 'icons', 'hand-finger-sm.png') )
+            icn = QIcon(os.path.join(utils.scriptdir, 'icons', 'hand-finger-sm.png'))
 
         r = QRect(5, 20, 12, 12)
         r.translate(option.rect.x(), option.rect.y())
-        painter.drawPixmap(r, icn.pixmap(QSize(12,12)))
+        painter.drawPixmap(r, icn.pixmap(QSize(12, 12)))
 
 
         # LATEST RUN
         if automaton.latest_run['timestamp']:
-            r = QRect(5, 52, option.rect.width()-10, 20)
+            r = QRect(5, 52, option.rect.width() - 10, 20)
             r.translate(option.rect.x(), option.rect.y())
             painter.drawText(r, Qt.AlignLeft, "Latest run: %s" % automaton.latest_run['timestamp'].strftime("%Y-%m-%d %H:%M:%S"))
-
             # r = option.rect.adjusted(100, 40, 0, 0)
             # painter.drawText(r.left(), r.top(), r.width(), r.height(), Qt.AlignLeft, automaton.latest_run['datetime'].strftime("%Y-%m-%d %H:%M:%S"))
-        
 
     def sizeHint(self, option, index):
         return QSize(200, 70)
 
 
 class Automaton(QStandardItem):
-    
+
     def __init__(self, *args, **kwargs):
         super(Automaton, self).__init__(*args, **kwargs)
-        
+
         self.setData(self, Qt.UserRole)
         self.watcher = QFileSystemWatcher()
-        self.timer = QTimer() 
-        
+        self.timer = QTimer()
+
         self.watch_window = {}
 
         self.latest_run = {}
         self.is_running = False
-        
+
         self.config = ConfigManager()
         self.config.set_defaults({
             'mode': MODE_WATCH_FOLDER,
-            'is_active':True,
+            'is_active': True,
             'trigger_hold': 1,
-            'notebook_paths':'',
+            'notebook_paths': '',
             'output_path': '{home}/{notebook_filename}_{datetime}_',
             'output_format': 'html',
 
-            'watched_files':[],
-            'watched_folder':'',
-            'watch_window':15,
+            'watched_files': [],
+            'watched_folder': '',
+            'watch_window': 15,
 
-            'loop_watched_folder':False,
+            'iterate_watched_folder': True,
+            'iterate_wildcard': '.csv',
 
-            'timer_seconds':60,
+            'timer_seconds': 60,
         })
 
         self.runner = None
         self.lock = None
 
         self.latest_run = {
-            'timestamp':None,
-            'success':None,
+            'timestamp': None,
+            'success': None,
         }
-        
+
         # Set up all the triggers
         self.watcher.fileChanged.connect(self.file_trigger_accumulator)
         self.watcher.directoryChanged.connect(self.trigger)
         self.timer.timeout.connect(self.trigger)
-        
+
     def startup(self):
+        if self.config.get('is_active') == False:
+            return False
+    
         if self.config.get('mode') == MODE_TIMER:
-            self.timer.setInterval( self.config.get('timer_seconds')*1000 )
+            self.timer.setInterval(self.config.get('timer_seconds') * 1000)
             self.timer.start()
-            
+
         elif self.config.get('mode') == MODE_WATCH_FILES:
-            current_paths = self.watcher.files() + self.watcher.directories() 
+            current_paths = self.watcher.files() + self.watcher.directories()
             if current_paths:
-                self.watcher.removePaths( current_paths )
+                self.watcher.removePaths(current_paths)
             self.watch_window = {}
-            self.watcher.addPaths( self.config.get('watched_files') )
+            self.watcher.addPaths(self.config.get('watched_files'))
 
         elif self.config.get('mode') == MODE_WATCH_FOLDER:
-            current_paths = self.watcher.files() + self.watcher.directories() 
+            current_paths = self.watcher.files() + self.watcher.directories()
             if current_paths:
-                self.watcher.removePaths( current_paths )
-            self.watcher.addPath( self.config.get('watched_folder') )
-        
+                self.watcher.removePaths(current_paths)
+            self.watcher.addPath(self.config.get('watched_folder'))
+
     def shutdown(self):
         if self.config.get('mode') == MODE_TIMER:
             self.timer.stop()
-            
-        elif self.config.get('mode') == MODE_WATCH_FILES or self.config.get('mode') == MODE_WATCH_FOLDER:
-            current_paths = self.watcher.files() + self.watcher.directories() 
-            if current_paths:
-                self.watcher.removePaths( current_paths )
-            self.watch_window = {}
 
+        elif self.config.get('mode') == MODE_WATCH_FILES or self.config.get('mode') == MODE_WATCH_FOLDER:
+            current_paths = self.watcher.files() + self.watcher.directories()
+            if current_paths:
+                self.watcher.removePaths(current_paths)
+            self.watch_window = {}
         
     def load_notebook(self, filename):
         try:
@@ -488,15 +480,15 @@ class Automaton(QStandardItem):
             return None
         else:
             return nb
-            
+
     def file_trigger_accumulator(self, f):
         # Accumulate triggers from changed files: if 3 files are specified to watch
         # we only want to fire once _all_ files have changed (within the given watch window)
         current_time = datetime.now()
-        self.watch_window[f] = current_time # timestamp
-        self.watch_window = {k:v for k,v in self.watch_window.items() if current_time - timedelta(seconds=self.config.get('watch_window')) < v}
-        
-        if set( self.watch_window.keys() ) == set( self.watcher.files() + self.watcher.directories() ):
+        self.watch_window[f] = current_time  # timestamp
+        self.watch_window = {k: v for k, v in self.watch_window.items() if current_time - timedelta(seconds=self.config.get('watch_window')) < v}
+
+        if set(self.watch_window.keys()) == set(self.watcher.files() + self.watcher.directories()):
             self.trigger()
 
     def trigger(self, e=None):
@@ -507,32 +499,35 @@ class Automaton(QStandardItem):
         if self.lock is None:
             self.is_running = True
             self.update()
-            self.lock = QTimer.singleShot(self.config.get('trigger_hold')*1000, self.run)
-            
+            self.lock = QTimer.singleShot(self.config.get('trigger_hold') * 1000, self.run)
+            # Shutdown so we don't get in infinite loops
+            self.shutdown()
+
     def run(self, vars={}):
 
         default_vars = {
             'home': os.path.expanduser('~'),
             'version': VERSION_STRING,
         }
-        
-        default_vars_and_config = dict( list( default_vars.items() ) + list( self.config.config.items() ) )
+
+        default_vars_and_config = dict(list(default_vars.items()) + list(self.config.config.items()))
 
         if self.runner == None:
             # Postpone init to trigger for responsiveness on add/remove
             self.runner = NotebookRunner(None, pylab=True, mpl_inline=True)
 
-        if self.config.get('mode') == MODE_WATCH_FOLDER and self.config.get('loop_watched_folder'):
+        if self.config.get('mode') == MODE_WATCH_FOLDER and self.config.get('iterate_watched_folder'):
             for (dirpath, dirnames, filenames) in os.walk(self.config.get('watched_folder')):
                 break
-            
+
+            filenames = [f for f in filenames if self.config.get('iterate_wildcard') in f]
             logging.info('Watched folder contains %d files; looping' % len(filenames))
             # Filenames contains the list of files in the folder
         else:
             filenames = [None]
 
         self.latest_run['timestamp'] = datetime.now()
-            
+
         try:
             for f in filenames:
                 now = datetime.now()
@@ -542,11 +537,11 @@ class Automaton(QStandardItem):
                     'time': now.time().strftime("%H.%M.%S"),
                     'filename': f,
                 }
-                vars = dict( list( default_vars_and_config.items() ) + list( current_vars.items() ) )
+                vars = dict(list(default_vars_and_config.items()) + list(current_vars.items()))
 
                 for nb_path in self.config.get('notebook_paths'):
                     nb = self.load_notebook(nb_path)
-        
+
                     if nb:
                         # Add currently running notebook path to vars
                         vars['notebook_path'] = nb_path
@@ -557,12 +552,12 @@ class Automaton(QStandardItem):
                         if parent_folder:
                             try:
                                 utils.mkdir_p(parent_folder)
-                            except: # Can't create folder
+                            except:  # Can't create folder
                                 self.latest_run['success'] = False
                                 raise
-    
+
                         self.run_notebook(nb, vars)
-                
+
                     else:
                         raise NotebookNotFound(nb_path)
 
@@ -571,15 +566,17 @@ class Automaton(QStandardItem):
             traceback.print_exc()
             exctype, value = sys.exc_info()[:2]
             logging.error("%s\n%s\n%s" % (exctype, value, traceback.format_exc()))
-        
+
         finally:
             self.is_running = False
             self.lock = None
             self.update()
+            # Restart
+            self.startup()            
 
     def run_notebook(self, nb, vars={}):
-        if len(nb['worksheets'])==0:
-            nb['worksheets']=[NotebookNode({'cells':[], 'metadata':{}})]
+        if len(nb['worksheets']) == 0:
+            nb['worksheets'] = [NotebookNode({'cells': [], 'metadata': {}})]
 
         start = nb['worksheets'][0]['cells']
         start.insert(0, Struct(**{
@@ -590,7 +587,7 @@ class Automaton(QStandardItem):
             'prompt_number': -1,
             'input': 'qtipy=%s' % vars,
             'metadata': {},
-        }) )
+        }))
         self.runner.nb = nb
 
         try:
@@ -600,26 +597,26 @@ class Automaton(QStandardItem):
             raise
         else:
             self.latest_run['success'] = True
-           
+
         ext = dict(
             html='html',
             slides='slides',
             latex='latex',
             markdown='md',
             python='py',
-            rst='rst',        
+            rst='rst',
         )
-            
+
         output, resources = IPyexport(IPyexporter_map[self.config.get('output_format')], self.runner.nb)
         output_path = vars['output_path'] + 'notebook.%s' % ext[self.config.get('output_format')]
         logging.info("Exporting updated notebook to %s" % output_path)
-        with open(output_path,"w") as f:
+        with open(output_path, "w") as f:
             f.write(output)
-            
 
     def update(self):
         global _w
-        _w.viewer.update( self.index())
+        _w.viewer.update(self.index())
+
 
 class MainWindow(QMainWindow):
 
@@ -630,22 +627,22 @@ class MainWindow(QMainWindow):
             'file': self.menuBar().addMenu(tr('&File')),
             'edit': self.menuBar().addMenu(tr('&Edit')),
             'control': self.menuBar().addMenu(tr('&Control')),
-        }        
-        
+        }
+
         t = self.addToolBar('File')
         t.setIconSize(QSize(16, 16))
 
         action = QAction(QIcon(os.path.join(utils.scriptdir, 'icons', 'folder-horizontal-open.png')), tr('Load automatons'), self)
-        action.setShortcut( QKeySequence.Open )
+        action.setShortcut(QKeySequence.Open)
         action.setStatusTip('Load automatons')
-        action.triggered.connect( self.load_automatons )
+        action.triggered.connect(self.load_automatons)
         t.addAction(action)
         self.menuBars['file'].addAction(action)
 
         action = QAction(QIcon(os.path.join(utils.scriptdir, 'icons', 'disk.png')), tr('Save automatons'), self)
-        action.setShortcut( QKeySequence.Save )
+        action.setShortcut(QKeySequence.Save)
         action.setStatusTip('Save automatons')
-        action.triggered.connect( self.save_automatons )
+        action.triggered.connect(self.save_automatons)
         t.addAction(action)
         self.menuBars['file'].addAction(action)
 
@@ -653,23 +650,22 @@ class MainWindow(QMainWindow):
         t.setIconSize(QSize(16, 16))
 
         action = QAction(QIcon(os.path.join(utils.scriptdir, 'icons', 'plus-circle.png')), tr('Add automaton'), self)
-        action.setShortcut( QKeySequence.New )
+        action.setShortcut(QKeySequence.New)
         action.setStatusTip('Add new automaton')
-        action.triggered.connect( self.add_new_automaton )
+        action.triggered.connect(self.add_new_automaton)
         t.addAction(action)
         self.menuBars['edit'].addAction(action)
 
         action = QAction(QIcon(os.path.join(utils.scriptdir, 'icons', 'property.png')), tr('Edit automaton'), self)
         action.setStatusTip('Edit automaton')
-        action.triggered.connect( self.edit_automaton )
+        action.triggered.connect(self.edit_automaton)
         t.addAction(action)
         self.menuBars['edit'].addAction(action)
 
-
         action = QAction(QIcon(os.path.join(utils.scriptdir, 'icons', 'cross.png')), tr('Delete automaton'), self)
-        action.setShortcut( QKeySequence.Delete )
+        action.setShortcut(QKeySequence.Delete)
         action.setStatusTip('Delete automaton')
-        action.triggered.connect( self.delete_automaton )
+        action.triggered.connect(self.delete_automaton)
         t.addAction(action)
         self.menuBars['edit'].addAction(action)
 
@@ -677,16 +673,18 @@ class MainWindow(QMainWindow):
         t.setIconSize(QSize(16, 16))
 
         action = QAction(QIcon(os.path.join(utils.scriptdir, 'icons', 'control.png')), tr('Enable'), self)
-        action.setShortcut(tr("Ctrl+E"));
+        action.setShortcut(tr("Ctrl+E"))
+        
         action.setStatusTip('Enable automaton')
-        action.triggered.connect( self.enable_automaton )
+        action.triggered.connect(self.enable_automaton)
         t.addAction(action)
         self.menuBars['control'].addAction(action)
 
         action = QAction(QIcon(os.path.join(utils.scriptdir, 'icons', 'control-pause.png')), tr('Pause'), self)
-        action.setShortcut(tr("Ctrl+W"));
+        action.setShortcut(tr("Ctrl+W"))
+        
         action.setStatusTip('Pause automaton')
-        action.triggered.connect( self.pause_automaton )
+        action.triggered.connect(self.pause_automaton)
         t.addAction(action)
         self.menuBars['control'].addAction(action)
 
@@ -695,12 +693,13 @@ class MainWindow(QMainWindow):
 
         btn = QToolButton(self)
         action = QAction(QIcon(os.path.join(utils.scriptdir, 'icons', 'play.png')), tr('Run now'), self)
-        action.setShortcut(tr("Ctrl+R"));
+        action.setShortcut(tr("Ctrl+R"))
+        
         btn.setText(tr('Run now'))
         btn.setStatusTip('Run now...')
         btn.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
         btn.setDefaultAction(action)
-        action.triggered.connect( self.run_automaton )
+        action.triggered.connect(self.run_automaton)
         t.addWidget(btn)
         #self.menuBars['control'].addAction(action)
 
@@ -712,35 +711,27 @@ class MainWindow(QMainWindow):
         self.viewer.setItemDelegate(AutomatonListDelegate(self.viewer))
         self.automatons = QStandardItemModel(self.viewer)
         self.viewer.setModel(self.automatons)
-        
-
 
         # Initiate logging
-        self.log = QTreeWidget()
-        self.log.setColumnCount(2)
-        self.log.expandAll()
-
-        self.log.setHeaderLabels(['ID', 'Message'])
-        self.log.setUniformRowHeights(True)
-        self.log.hideColumn(0)
+        self.log = QTextEdit()
+        self.log._current_rows = []
+        self.log.setReadOnly(True)
 
         logHandler = Logger(self, self.log)
         logging.getLogger().addHandler(logHandler)
         logging.info('Welcome to QtIPy')
 
+        self.tabs.addTab(self.viewer, 'Automatons')
+        self.tabs.addTab(self.log, 'Log')
 
-        self.tabs.addTab(self.viewer,'Automatons')
-        self.tabs.addTab(self.log,'Log')
-
-        self.setCentralWidget( self.tabs )
-
+        self.setCentralWidget(self.tabs)
 
         self.setWindowTitle("QtIPy: The data automator")
         self.statusBar().showMessage(tr('Ready'))
 
-        self.setMinimumSize( QSize(400,500) )
+        self.setMinimumSize(QSize(400, 500))
         self.show()
-        
+
     def add_new_automaton(self):
         '''
         Define a new automaton and add to the list
@@ -753,25 +744,25 @@ class MainWindow(QMainWindow):
         '''
         automaton = Automaton()
         self.automatons.appendRow(automaton)
-        lastitem = self.automatons.item( self.automatons.rowCount()-1 )
-        self.viewer.setCurrentIndex( lastitem.index() )
+        lastitem = self.automatons.item(self.automatons.rowCount() - 1)
+        self.viewer.setCurrentIndex(lastitem.index())
         self.edit_automaton()
-        
+
     def edit_automaton(self):
         '''
         
         
         '''
         try:
-            automaton = self.automatons.itemFromIndex( self.viewer.selectionModel().currentIndex() )
+            automaton = self.automatons.itemFromIndex(self.viewer.selectionModel().currentIndex())
         except:
             return
-        
+
         automaton.shutdown()
         dlg = AutomatonDialog(self)
-        dlg.config.set_many( automaton.config.config )
+        dlg.config.set_many(automaton.config.config)
         if dlg.exec_():
-            automaton.config.set_many( dlg.config.config )
+            automaton.config.set_many(dlg.config.config)
             if automaton.config.get('is_active'):
                 automaton.startup()
             automaton.update()
@@ -781,16 +772,16 @@ class MainWindow(QMainWindow):
         '''
         _btn = QMessageBox.question(self, "Confirm delete", "Are you sure you want to delete this automaton?")
         if _btn == QMessageBox.Yes:
-            automaton = self.automatons.itemFromIndex( self.viewer.selectionModel().currentIndex() )
+            automaton = self.automatons.itemFromIndex(self.viewer.selectionModel().currentIndex())
             automaton.shutdown()
             automaton_idx = self.viewer.selectionModel().selectedIndexes()[0]
-            self.automatons.removeRows( automaton_idx.row(), 1, QModelIndex() )
-        
+            self.automatons.removeRows(automaton_idx.row(), 1, QModelIndex())
+
     def enable_automaton(self):
         '''
         '''
         try:
-            automaton = self.automatons.itemFromIndex( self.viewer.selectionModel().currentIndex() )
+            automaton = self.automatons.itemFromIndex(self.viewer.selectionModel().currentIndex())
         except:
             return
         automaton.config.set('is_active', True)
@@ -801,7 +792,7 @@ class MainWindow(QMainWindow):
         '''
         '''
         try:
-            automaton = self.automatons.itemFromIndex( self.viewer.selectionModel().currentIndex() )
+            automaton = self.automatons.itemFromIndex(self.viewer.selectionModel().currentIndex())
         except:
             return
         automaton.config.set('is_active', False)
@@ -812,32 +803,29 @@ class MainWindow(QMainWindow):
         '''
         '''
         try:
-            automaton = self.automatons.itemFromIndex( self.viewer.selectionModel().currentIndex() )
+            automaton = self.automatons.itemFromIndex(self.viewer.selectionModel().currentIndex())
         except:
             return
         automaton.trigger(None)
-    
+
     def load_automatons(self):
         '''
         '''
         global _w
         filename, _ = QFileDialog.getOpenFileName(_w, "Load QtIPy Automatons", '', "QtIPy Automaton File (*.qifx);;All files (*.*)")
         if filename:
-        
+
             self.automatons.clear()
-            
+
             tree = et.parse(filename)
             automatons = tree.getroot()
-        
+
             for automatonx in automatons.findall('Automaton'):
                 automaton = Automaton()
                 automaton.config.setXMLConfig(automatonx)
-            
+
                 self.automatons.appendRow(automaton)
-                
-                if automaton.config.get('is_active'):
-                    automaton.startup()
-            
+                automaton.startup()
             
     def save_automatons(self):
         '''
@@ -845,22 +833,23 @@ class MainWindow(QMainWindow):
         global _w
         filename, _ = QFileDialog.getSaveFileName(_w, "Save QtIPy Automatons", '', "QtIPy Automaton File (*.qifx);;All files (*.*)")
         if filename:
-        
+
             root = et.Element("QtIPy")
             root.set('xmlns:mpwfml', "http://martinfitzpatrick.name/schema/QtIPy/2013a")
 
             # Build a JSONable object representing the entire current workspace and write it to file
             for i in range(0, self.automatons.rowCount()):
                 a = self.automatons.item(i)
-            
+
                 automaton = et.SubElement(root, "Automaton")
                 automaton = a.config.getXMLConfig(automaton)
 
             tree = et.ElementTree(root)
             tree.write(filename)  # , pretty_print=True)
-                            
+
     def sizeHint(self):
-        return QSize(400,500)     
+        return QSize(400, 500)
+
 
 def main():
     # Create a Qt application
@@ -878,7 +867,7 @@ def main():
     logging.info('Ready.')
     app.exec_()  # Enter Qt application main loop
     logging.info('Exiting.')
-    
+
     sys.exit()
 
 if __name__ == "__main__":
